from ladon.exceptions.base import LadonException

class LadonizeException(LadonException):
	def __init__(self,sinfo,method):
		LadonException.__init__(self)
		self.sinfo = sinfo
		self.method = method
		
	def longstr(self):
		return 'Method "%s" in service "%s" [%s:%d]' % (self.method,self.sinfo.servicename,self.sinfo.sourcefile,self.sinfo.lineno)
	def shortstr(self):
		return '%s::%s' % (self.sinfo.servicename,self.method)


class ArgDefCountMismatch(LadonizeException):
	"""Exception is raised if the number of argument type defines passed to the ladonize
	decorator does not match the number of arguments that the method takes.
	example:
	
	@ladonize(String,Int,_rtype=String)
	def adduser(username,id,password):
		...
		implementation
		...
		
	In the example above only 2 arguments have been defined in the ladonize decorator, but
	the method takes 3.
	"""
	def __init__(self,sinfo,method,ladonize_argtypes,method_argnames):
		LadonizeException.__init__(self,sinfo,method)
		self.ladonize_argtypes = ladonize_argtypes
		self.method_argnames = method_argnames
	
	def __str__(self):
		return 'Mismatch between the number of argument type defines (%d) and the actual number of arguments in "%s" (%d)' % \
			(len(self.ladonize_argtypes),self.shortstr(),len(self.method_argnames))


class ArgTypeMismatch(LadonizeException):
	"""Exception is raised when a ladonized method is called with arguments that do not
	match the pre-defined argument types.
	"""
	def __init__(self,sinfo,method,argname,expected_type,recieved_type):
		LadonizeException.__init__(self,sinfo,method)
		self.argname = argname
		self.expected_type = expected_type
		self.recieved_type = recieved_type
	
	def __str__(self):
		return 'Type mismatch while passing argument "%s" to %s: Expected %s recieved %s' % \
			(self.argname,self.shortstr(),self.expected_type,self.recieved_type)


class DefaultArgTypeMismatch(LadonizeException):
	"""Exception is raised when a ladonized method has default values that do not
	match the pre-defined argument types.
	"""
	def __init__(self,sinfo,method,argname,expected_type,recieved_type):
		LadonizeException.__init__(self,sinfo,method)
		self.argname = argname
		self.expected_type = expected_type
		self.recieved_type = recieved_type
	
	def __str__(self):
		return 'Default type mismatch on argument "%s" in %s: Expected %s recieved %s' % \
			(self.argname,self.shortstr(),self.expected_type,self.recieved_type)


class ReturnTypeUndefined(LadonizeException):
	"""Exception is raised when a ladonized method is called with arguments that do not
	match the pre-defined argument types.
	"""
	def __init__(self,sinfo,method):
		LadonizeException.__init__(self,sinfo,method)
	
	def __str__(self):
		return 'Ladonized method (%s) must have a return type defined (keyword: "_rtype")' % \
			(self.shortstr())


class ReturnTypeMismatch(LadonizeException):
	"""Exception is raised when a ladonized method is called with arguments that do not
	match the pre-defined argument types.
	"""
	def __init__(self,sinfo,method,expected_rtype,recieved_rtype):
		LadonizeException.__init__(self,sinfo,method)
		self.expected_rtype = expected_rtype
		self.recieved_rtype = recieved_rtype
	
	def __str__(self):
		return 'Return-type mismatch in %s: Expected %s recieved %s' % \
			(self.shortstr(),self.expected_rtype,self.recieved_rtype)


class NeedToDefineException(LadonException):
	def __init__(self,text):
		LadonException.__init__(self)
		self.txt = text
	
	def __str__(self):
		return self.txt



# -*- coding: utf-8 -*-

from ladon.exceptions.types import *
from ladon.types import get_type_info
import inspect,sys,re
from ladon.types.attachment import attachment,extract_attachment_reference
from ladon.compat import PORTABLE_BYTES,PORTABLE_STRING

rx_cid = re.compile('^cid:(.+)$',re.I)

class LadonType(object):
	"""LadonType must be specialized by all complex service types. It tells the Ladon
	framework that it should expect multiple primitives and/or lists to be defined as
	class attributes. Using LadonType it is possible to build nested types.

	LadonType specializations must define it's attributes as types (either primitive
	types, other LadonType specialization types, lists of primitive or lists of 
	LadonType specialization). This explains the Ladon framework what to expect. There
	are 5 primitive types:
		
		+------------+------------+
		| Python 2   | Python 3   |
		+============+============+
		| str        | bytes      |
		+------------+------------+
		| unicode    | str        |
		+------------+------------+
		| bool       | bool       |
		+------------+------------+
		| float      | float      |
		+------------+------------+
		| int        | int        |
		+------------+------------+
		| attachment | attachment |
		+------------+------------+
	"""
	
	def __init__(self,prime_dict=None,tc=None,export_dict=None):
		global rx_cid
		if type(prime_dict)==dict:
			type_dict = get_type_info(self.__class__)
			for attr_name,attr_type,props in type_dict['attributes']:
				if type(attr_type)==list:
					# Assumption list attributes are always optional
					attr_val = []
					if attr_name in prime_dict:
						if prime_dict[attr_name]==None:
							# Null lists are accepted ans parsed as empty list
							prime_dict[attr_name] = []
						elif type(prime_dict[attr_name])!=list:
							raise ListAttributeMismatch(
								self,prime_dict,
								'Expected list type for attribute "%s" got "%s"' % \
								(attr_name,str(type(prime_dict[attr_name]))),
								attr_name)
						type_info = get_type_info(attr_type[0])
						try:
							if type_info:
								for item in prime_dict[attr_name]:
									attr_val += [attr_type[0](prime_dict=item,tc=tc,export_dict=export_dict)]
							elif attr_type[0]==attachment:
								for item in prime_dict[attr_name]:
									try:
										attr_val += [extract_attachment_reference(item,export_dict,tc.encoding)]
									except Exception as e:
										raise NonExistingAttachment(
											self,prime_dict,
											str(e),
											attr_name)
							else:
								for item in prime_dict[attr_name]:
									attr_val += [tc.from_unicode_string(item,attr_type[0])]
						except Exception as e:
							raise SubitemTypeMismatch(
								self,prime_dict,
								'Type mismatch for subitem of attibute "%s", expected "%s" got "%s"\nDetails:%s' % \
								(attr_name,str(attr_type[0]),str(item),str(e)),
								attr_name)
				else:
					filters = props.get('filters')
					# Description of incoming filters:
					# PORTABLE_BYTES = Python 2: str, Python 3: bytes
					# * 'incoming_raw': 
					#     list functions that recieve the raw value (PORTABLE_BYTES). The function can for
					#     instance do business logic validation and raise a client fault if the value makes
					#     no sense.
					#     Or it can return a new value (PORTABLE_BYTES) and thereby acting as a raw modifier.
					# * 'incoming': 
					#     list functions that recieve the value in the service specified type.
					#     The function can do validation raise a client fault if the value makes no sense.
					#     Or it can return a new value in the service specified type acting as a modifier.
					prefilters = None
					postfilters = None
					if filters and filters.get('incoming_raw'):
						prefilters = filters.get('incoming_raw')
					if filters and filters.get('incoming'):
						postfilters = filters.get('incoming')

					# Need some kind of check for optional in type definition class
					if attr_name in prime_dict:
						type_info = get_type_info(attr_type)
						val = prime_dict[attr_name]
						
						if val==None and props.get('nullable')==True:
							attr_val = val
							## Run filters on incoming value if value was None
							#if filters and filters.get('incoming'):
								#for func in filters.get('incoming'):
									#attr_val = func(attr_val)
						elif type_info:
							attr_val = attr_type(prime_dict=val,tc=tc,export_dict=export_dict)
						elif attr_type==attachment:
							try:
								attr_val = extract_attachment_reference(val,export_dict,tc.encoding)
							except Exception as e:
								raise NonExistingAttachment(
									self,prime_dict,
									str(e),
									attr_name)
						else:
							attr_val = tc.from_unicode_string(val,attr_type,prefilters,postfilters)
					else:
						attr_val=props.get('default')
						if attr_val==None and props.get('nullable')!=True:
							raise MandatoryAttributeMissing(
								self,prime_dict,
								'Missing prime_dict attribute is not nullable "%s"' % attr_name,
								attr_name)
						## Run filters on incoming value if value was None
						#if filters and filters.get('incoming'):
							#for func in filters.get('incoming'):
								#attr_val = func(attr_val)

				setattr(self,attr_name,attr_val)
		elif prime_dict:
			type_dict = TypeManager.global_type_dict[self.__class__]
			raise LadonTypePrimerMismatch(
				self,prime_dict,
				'Dictionary expected for prime_dict got "%s" of value "%s"' % \
				(type(prime_dict),
				str(prime_dict)))
			

	def __dict__(self,tc,response_attachments=None):
		"""
		Convert LadonType to dict representation. Argument tc is a TypeConverter
		which holds information about string conversions and whether unsafe conversions
		are allowed or not.
		"""
		
		res_dict = {}
		type_dict = TypeManager.global_type_dict[self.__class__]
		for attr_name,attr_type,props in type_dict['attributes']:
			if type(attr_type)==list:
				# Assumption list attributes are always optional
				try:
					attr_val = getattr(self,attr_name)
				except:
					# assumed optional, so this is ok
					continue
				if not type(attr_val)==list:
					raise ListExpected(
						self,
						'Expected list got: %s' % type(attr_val),
						attr_name)
				if attr_type == attr_val:
					# attribute not touched and still exposing the type
					continue
				
				attr_list = []
				res_dict[attr_name] = attr_list
				
				is_ladontype_items = False
				if attr_type[0] in TypeManager.global_type_dict:
					is_ladontype_items = True
				
				for val in attr_val:
					if is_ladontype_items:
						attr_list += [val.__dict__(tc,response_attachments)]
					elif attr_type[0] == attachment:
						if not type(val) == attachment:
							raise AttachmentExpected(
								self,
								'Expected list of attachments got item of type: %s' % type(val),
								attr_name)
						attr_list += [response_attachments.add_attachment(val)]
					else:
						attr_list += [tc.to_unicode_string(val,attr_type[0])]
			else:
				try:
					attr_val = getattr(self,attr_name)
				except:
					if props.get('nullable')==True:
						attr_val = props.get('default')
					else:
						# Assumption non-list non nullable attributes are not optional
						raise NeedToDefineParseTimeException('Non-optional attribute missing in LadonType while serializing')

				filters = props.get('filters')
				# Description of outgoing filters:
				# PORTABLE_STRING = Python 2: unicode, Python 3: str
				# * 'outgoing': 
				#     list functions that recieve the value in the service specified type.
				#     The function can do validation raise a server fault if the value makes no sense.
				#     Or it can return a new value in the service specified type acting as a modifier.
				# * 'outgoing_raw': 
				#     list functions that recieve the raw value (PORTABLE_STRING). The function can for
				#     instance do business logic validation and raise a server fault if the value makes
				#     no sense.
				#     Or it can return a new value (PORTABLE_STRING) and thereby acting as a raw modifier.
				prefilters = None
				postfilters = None
				if filters and filters.get('outgoing'):
					prefilters = filters.get('outgoing')
				if filters and filters.get('outgoing_raw'):
					postfilters = filters.get('outgoing_raw')

				if attr_val==None:
					# Attribute is None, this is OK if the attribute is
					# nullable.
					if props.get('nullable')==True:
						res_dict[attr_name] = attr_val
					else:
						# Attribute is not nullable
						raise NeedToDefineParseTimeException('Attribute "%s" (class: "%s") is not nullable' % (attr_name,(self.__class__)))
				elif isinstance(attr_val,dict) and props.get('type')==attr_type:
					# Attribute is dictionary type defined and untouched set default value
					# if available.
					attr_val = props.get('default')
					if attr_val==None and props.get('nullable')!=True:
						# Attribute is not nullable
						raise NeedToDefineParseTimeException('Attribute "%s" (class: "%s") is not nullable' % (attr_name,(self.__class__)))
					res_dict[attr_name] = attr_val
				elif attr_type in TypeManager.global_type_dict:
					try:
						res_dict[attr_name] = attr_val.__dict__(tc,response_attachments)
					except Exception as e:
						raise AttributeConversionException(
							self,
							'An exception was raised while processing LadonType attribute "%s"\nDetails:%s' % (attr_name,str(e)),
							attr_name)
				elif attr_type == attachment:
					if not type(attr_val) == attachment:
						raise AttachmentExpected(
							self,
							'Expected attachment got %s' % type(attr_val),
							attr_name)
					res_dict[attr_name] = response_attachments.add_attachment(attr_val)
				else:
					try:
						res_dict[attr_name] = tc.to_unicode_string(attr_val,attr_type,prefilters,postfilters)
					except Exception as e:
						raise AttributeConversionException(
							self,
							'An exception was raised while processing attribute "%s"\nDetails:%s' % (attr_name,str(e)),
							attr_name)
		return res_dict

from ladon.types.typemanager import TypeManager



# -*- coding: utf-8 -*-

from ladon.compat import PORTABLE_STRING

catalog_default_template = PORTABLE_STRING("""
<!DOCTYPE html>
<html>
	<head>
		<meta content="text/html; charset={{ charset }}" http-equiv="Content-Type" />
		<title>{{ catalog_name }}</title>
		<style>
{{ css }}
{{ extra_css }}
		</style>
	</head>
	<body>
		<div class="catName">{{ catalog_name }}</div>
		<div class="catDesc">{{ catalog_desc }}</div>
		<div class="catContent">
			<ul class="catService">
{% for service in services %}
				<li>
					<b><a href="{{ client_path }}/{{ service.servicename }}{{ '?' if query_string }}{{query_string}}">{{ service.servicename }}</a></b><br/>
					{{ service.doc }}
				</li>
{% endfor %}
			</ul>
		</div>
		<div class="catGen">Powered by Ladon for Python</div>
	</body>
</html>
""")

service_default_template = PORTABLE_STRING("""
<html>
	<head>
		<meta content="text/html; charset={{ charset }}" http-equiv="Content-Type" />
		<title>{{ servicename }}</title>
		<style>
{{ css }}
		</style>
	</head>
	<body>
		<div class="serviceName">
			<div class="title">{{ servicename }}</div>
			<form method="get" class="skin-selector">
				<label for="skin-select">skins:</label>
				<select id="skin-select" name="skin" onchange="document.forms[0].submit()">
					<option value="">Default</option>
					{% for skin in skins %}
					<option value="{{skin}}"{% if skin == current_skin %} selected{% endif %}>{{ skin|title }}</option>
					{% endfor %}
				</select>
			</form>
		</div>
		<div class="serviceDesc">
			<span class="descriptionHead">Service Description:</span>
			<div class="docLines">
				{{ doc }}
			</div>
		</div>
		<div class="serviceInterfaces">
			<span class="interfaceHead">Available Interfaces:</span>
			<ul class="interfaceList">
{% for interface in interfaces %}
				<li>{{ interface }} [ <a href="{{ client_path }}/{{ interface }}">url</a> <a href="{{ client_path }}/{{ interface }}/description">description</a> ]</li>
{% endfor %}
			</ul>
		</div>
		<div class="serviceDocumentation">
			<span class="methodsHead">Methods:</span>
			<ul class="methodList">
{% for method in methods %}
				<li class="methodEntry">
					<div class="methodDef">
						<span class="methodName">{{ method.methodname }}</span>
						(
	{% set sep = '' %}
	{% for param in method.params %}
						{{ sep }} 
						<span class="paramType">
		{% if param.ladontype %}
							<a href="#{{ param.ladontype }}">{{ param.type }}</a>
		{% else %}
							{{ param.type }}
		{% endif %}
						</span> 
						<span class="paramName">{{ param.name }}</span>
		{% set sep = ',' %}
	{% endfor %}
						)
					</div>
					<div class="docLines">
						{{ method.doc }}
					</div>
	{% for param in method.params %}
		{% if loop.first %}
					<u>Paramters:</u>
					<ul>
		{% endif %}
						<li class="paramDesc">
							<span class="paramName">{{ param.name }}</span>: <span class="paramType">
		{% if param.ladontype %}
								<a href="#{{ param.ladontype }}">{{ param.type }}</a>
		{% else %}
								{{ param.type }}
		{% endif %}
							</span>
		{% if param.optional %}
								[ default: {{ param.default }} ]
		{% endif %}
							<br/>
							<div class="docLines">
								{{ param.doc }}
							</div>
						</li>
		{% if loop.last %}
					</ul>
		{% endif %}
	{% endfor %}
					<u>Returns:</u>
					<span class="paramType">
	{% if method.returns.ladontype %}
						<a href="#{{ method.returns.ladontype }}">{{ method.returns.type }}</a>
	{% else %}
						{{ method.returns.type }}
	{% endif %}
					</span>
					<div class="docLines">
						{{ method.returns.doc }}
					</div>
				</li>
{% endfor %}
			</ul>
			<span class="TypesHead">Types:</span>
			<ul>
{% for type in types %}
				<li>
					<div class="typeDef"><a name="{{ type.name }}"><span class="typeName">{{ type.name }}</span></a></div>
					<br/>
					<u>Attributes:</u>
					<ul>
	{% for k,v in type.attributes.items() %}
						<li class="typeAttrib">
							<span class="paramName">{{ k }}</span>:
							<span class="paramType">
		{% if v.ladontype %}
							<a href="#${v.ladontype}">{{ v.type }}</a>
		{% else %}
								{{ v.type }}
		{% endif %}
							</span>
						</li>
	{% endfor %}
					</ul>
				</li>
{% endfor %}
			</ul>
		</div>
		<div class="catGen">Powered by Ladon for Python</div>
	</body>
</html>
""")



# -*- coding: utf-8 -*-
from ladon.ladonizer import ladonize
from ladon.types.ladontype import LadonType
from ladon.compat import PORTABLE_STRING
import pickle
from os.path import abspath, dirname, join
f = open(join(dirname(abspath(__file__)),'albums.pickle'),'rb')
albums = pickle.loads(f.read())
f.close()

class Band(LadonType):
	def __init__(self):
		self.name = 'bla'

	name = PORTABLE_STRING
	album_titles = [ PORTABLE_STRING ]

class Album(LadonType):
	band = Band
	title = PORTABLE_STRING
	songs = [ PORTABLE_STRING ]


class AlbumService(object):
	"""
	Search through albums and bands.
	"""
	
	@ladonize(PORTABLE_STRING,rtype=[ Album ])
	def listAlbums(self,search_frase=PORTABLE_STRING('')):
		"""
		Fetch a list of albums matching search_frase
		"""
		global albums
		album_list = []
		for band_name,albums_dict in albums.items():
			b = Band()
			#b.name = band_name
			b.name = None
			b.album_titles = []
			for album_title,songs in albums_dict.items():
				b.album_titles += [ album_title ]
				if len(search_frase)==0 or album_title.find(search_frase)>-1:
					a = Album()
					a.band = b
					a.title = album_title
					a.songs = [ ]
					for idx,song_title in songs:
						a.songs += [song_title]
					album_list += [a]
		return album_list

	@ladonize(PORTABLE_STRING,rtype=[ Band ])
	def listBands(self,search_frase=PORTABLE_STRING('')):
		"""
		Fetch a list of albums matching search_frase
		"""
		global albums
		bands = []
		for band_name,albums_dict in albums.items():
			if len(search_frase)==0 or band_name.find(search_frase)>-1:
				b = Band()
				b.name = band_name
				b.album_titles = []
				for album_title,songs in albums_dict.items():
					b.album_titles += [ album_title ]
				bands += [b]
		return bands



from ladon.ladonizer import ladonize

class Calculator(object):
  """
  This service does the math, and serves as example for new potential Ladon users.
  """
  @ladonize(int,int,rtype=int)
  def add(self,a,b):
    """
    Add two integers together and return the result

    @param a: 1st integer
    @param b: 2nd integer
    @rtype: The result of the addition
    """
    return a+b




# -*- coding: utf-8 -*-
import sys,imp,os,inspect
from ladon.exceptions.interfaces import *
from ladon.interfaces.base import BaseInterface
from ladon.compat import PORTABLE_STRING_TYPES

_interfaces = {}

#def install(mod,path=None):
	#global _exposers
	#if [str,unicode].count(type(path)):
		#search_path += [path]
	#elif [list,tuple].count(type(path)):
		#search_path += list(path)
	#search_path = sys.path + [os.path.dirname(__file__)]
	#fp, pathname, description = imp.find_module(mod,search_path)
	#if _exposers.has_key(pathname):
		#return
	#try:
		#impmod = imp.load_module(mod,fp,pathname,description)
		#_exposers[pathname] = impmod
	#except Exception, e:
		#raise ImportExposerException(mod,pathname,str(e))


def expose(class_type):
	"""
	This is a decorator for service interfaces. It basically checks that
	the interface classes which are exposed meets the requirements for
	interface implementations.
	It adds the interface to a global interface-cache.
	All default ladon interfaces are added using this decorator, if you are
	extending Ladon with a new interface use this decorator.
	Requirements:
	------------
	Interfaces must inherit the ladon.interfaces.base.BaseInterface class.
	"""
	global _interfaces
	if inspect.getmro(class_type).count(BaseInterface):
		if_names = class_type._interface_name()
		if type(if_names) in PORTABLE_STRING_TYPES:
			if_names = [ if_names ]
		if type(if_names) not in [list,tuple]:
			raise ExposeInterfaceException(class_type.__name__,"wefew",'Interfaces must implement the static _interface_name method to return a unique string or list of strings identifying the interface.')
		for if_name in if_names:
			if type(if_name) not in PORTABLE_STRING_TYPES:
				raise ExposeInterfaceException(class_type.__name__,"wefew",'Interfaces must implement the static _interface_name method to return a unique string or list of strings identifying the interface.')
			if if_name in _interfaces:
				raise ExposeInterfaceException(class_type.__name__,"wefwe",'The interface name is already being used by anothere interface.')
			_interfaces[if_name] = class_type
	return class_type


def accept_basetype(typ):
	global _interfaces
	result_interfaces = []
	for k,v in _interfaces.items():
		if v._accept_basetype(typ):
			result_interfaces += [v]
	return result_interfaces

def accept_list():
	global _interfaces
	result_interfaces = []
	for k,v in _interfaces:
		if v._accept_list():
			result_interfaces += [v]
	return result_interfaces

def name_to_interface(ifname):
	global _interfaces
	if ifname in _interfaces:
		return _interfaces[ifname]
	return None


import ladon.interfaces.soap
import ladon.interfaces.soap11
import ladon.interfaces.jsonwsp
import ladon.interfaces.jsonrpc10
import ladon.interfaces.xmlrpc



# -*- coding: utf-8 -*-

from ladon.exceptions.base import LadonException

class ExposerException(LadonException):
	def __init__(self,name,path):
		LadonException.__init__(self)
		self.name = name
		self.path = path

class ExposeInterfaceException(ExposerException):
	def __init__(self,name,path,error_str):
		ExposerException.__init__(self,name,path)
		self.error_str = error_str

	def __str__(self):
		return 'An error occured while exposing the Ladon interface "%s" at "%s". %s' % \
			(self.name,self.path,self.error_str)



# -*- coding: utf-8 -*-
from ladon.ladonizer import ladonize
from ladon.types.ladontype import LadonType
from ladon.compat import PORTABLE_STRING

class Grocery(LadonType):
	item_name = PORTABLE_STRING
	price = float

class Purchase(LadonType):
	grocery = Grocery
	quantity = int

class GroceryList(LadonType):
	purchases = [ Purchase ]

class ShopService(object):
	
	@ladonize(GroceryList,rtype=int)
	def addToBasket(self,glist):
		return 1



# -*- coding: utf-8 -*-

"""XMLRPC-Interface for ladon.
"""

__author__ = 'Dennis Ploeger <develop@dieploegers.de>'

import sys
if sys.version_info[0]==2:
	from StringIO import StringIO
elif sys.version_info[0]>=3:
	from io import StringIO

import base64
import re
import datetime
import traceback
from ladon.exceptions.dispatcher import UndefinedServiceMethod
from ladon.exceptions.service import ClientFault

from ladon.interfaces import expose

from ladon.interfaces.base import BaseInterface, \
    ServiceDescriptor, BaseRequestHandler, BaseResponseHandler, BaseFaultHandler
from ladon.compat import pytype_support, BytesIO, PORTABLE_STRING

from xml.dom import Node

from xml.dom.minidom import parseString, getDOMImplementation

def u(instring):
    if sys.version_info[0]==2:
        return PORTABLE_STRING(instring, 'utf-8')
    else:
        return PORTABLE_STRING(instring)

def is_binary(test_string):
    """Return true if the given filename is binary.
    @attention: based on http://bytes
    .com/topic/python/answers/21222-determine-file-type-binary-text on 6/08/2010
    @author: Trent Mick <TrentM@ActiveState.com>
    @author: Jorge Orpinel <jorge@orpinel.com>"""
    fin = StringIO(test_string)
    try:
        CHUNKSIZE = 1024
        while 1:
            chunk = fin.read(CHUNKSIZE)
            if '\0' in chunk: # found null byte
                return True
            if len(chunk) < CHUNKSIZE:
                break # done
    finally:
        fin.close()

    return False

class XMLRPCServiceDescriptor(ServiceDescriptor):
    """Generate XRDL (based on
        http://code.google.com/p/xrdl/source/browse/documentation/xrdl.xsd)"""

    _content_type = 'text/xml'
    _special_types = []

    def _get_type_name(self, type_class):

        try:

            return type_class.__name__

        except:

            return type(type_class).__name__

    def _type_to_xmlrpc(self, type_name):

        if type_name in ['str', 'unicode']:

            return 'string'

        elif type_name in ['int', 'long']:

            return 'int'

        elif type_name in ['float']:

            return 'double'

        elif type_name == 'datetime.time':

            return 'dateTime8601'

        elif type_name == 'bool':

            return 'boolean'

        elif type_name in self._special_types:

            return type_name

        elif type_name in ['list', 'tuple', 'set']:

            return 'array'

        elif type_name == 'dict':

            return 'struct'

        else:

            # Assume binary if no other type matches

            return 'base64'

    def generate(
        self,
        servicename,
        servicenumber,
        typemanager,
        methodlist,
        service_url,
        encoding
    ):

        type_dict = typemanager.type_dict

        for type_class in type_dict:

            self._special_types.append(self._get_type_name(type_class))

        impl = getDOMImplementation()

        resp_doc = impl.createDocument(None, 'service', None)

        resp_doc.documentElement.setAttribute(
            'url',
            service_url
        )

        resp_doc.documentElement.setAttribute(
            'ns',
            ''
        )

        resp_doc.documentElement.setAttribute(
            'name',
            servicename
        )

        # Types

        types_el = resp_doc.createElement('types')

        for type_class, type_info in type_dict.iteritems():
            type_el = resp_doc.createElement('type')

            type_el.setAttribute('name', type_info['name'])

            for member in type_info['attributes']:

                (member_name, member_type, member_opt) = member

                member_el = resp_doc.createElement('member')

                member_el.setAttribute(
                    'type',
                    self._type_to_xmlrpc(
                        self._get_type_name(member_type)
                    )
                )

                member_el.appendChild(
                    resp_doc.createTextNode(member_name)
                )

                type_el.appendChild(member_el)

            types_el.appendChild(type_el)

        resp_doc.documentElement.appendChild(types_el)

        # Methods

        methods_el = resp_doc.createElement('methods')

        for method in methodlist:

            method_info = method.serialize()

            method_el = resp_doc.createElement('method')

            method_el.setAttribute('name', method.name())
            method_el.setAttribute(
                'result',
                self._type_to_xmlrpc(
                    self._get_type_name(method_info['rtype'][0])
                )
            )

            for param in method.args():

                param_el = resp_doc.createElement('param')

                param_el.setAttribute(
                    'type',
                    self._type_to_xmlrpc(
                        self._get_type_name(param['type'])
                    )
                )

                param_el.appendChild(
                    resp_doc.createTextNode(param['name'])
                )

                method_el.appendChild(param_el)

            methods_el.appendChild(method_el)

        resp_doc.documentElement.appendChild(methods_el)

        return resp_doc.toxml(encoding = encoding)

class XMLRPCRequestHandler(BaseRequestHandler):

    def get_param_value(self, node):
        """Turn a param node into a value
        """

        type_defined = False

        for node_index in range(0, node.childNodes.length):

            if node.childNodes.item(node_index).nodeType != Node.TEXT_NODE:
                type_defined = True

                type_node = node.childNodes.item(node_index)

                current_type = type_node.tagName

        if not type_defined:
            # No type give, assume String

            current_type = "string"

        if current_type in ['i4', 'int']:
            return int(type_node.firstChild.data.strip())
        elif current_type == 'boolean':
            return int(type_node.firstChild.data.strip()) == 1
        elif current_type == 'double':
            return float(type_node.firstChild.data.strip())
        elif current_type in ['dateTime.iso8601', 'base64', 'string']:
            return type_node.firstChild.data.strip()

        # We have a list data type

        if current_type == 'struct':
            return_dict = {}

            members = type_node.getElementsByTagName('member')

            for member_index in range(0, members.length):

                current_member = members.item(member_index)

                name_node = current_member.getElementsByTagName('name')

                if name_node.length > 1:
                    raise ClientFault('More than one name nodes in a struct')

                if name_node.firstChild.nodeType != Node.TEXT_NODE:
                    raise ClientFault(
                        'Unexpected Node type %d while parsing '
                        'a struct member name' %
                            name_node.firstChild.nodeType
                    )

                key = name_node.firstChild.data.strip()

                value_node = current_member.getElementsByTagName('value')

                value = self.get_param_value(value_node)

                return_dict[key] = value

            return return_dict

        if current_type == 'array':

            return_list = []

            values = type_node.getElementsByTagName('value')

            for value_index in range(0, values.length):

                return_list.append(
                    self.get_param_value(values.item(value_index))
                )

            return return_list

    def parse_request(self,req,sinfo,encoding):

        req_dict = {'args': {}}

        req_doc = parseString(req)

        # Find method name

        method_name = req_doc.getElementsByTagName('methodName')

        req_dict['methodname'] = method_name.item(0).firstChild.data.strip()

        # Fill params

        if sinfo.method(req_dict['methodname']) is None:

            # Unknown method

            raise UndefinedServiceMethod(
                'xmlrpc',
                sinfo.servicename,
                'Unknown method %s' % req_dict['methodname']
            )

        args = sinfo.method(req_dict['methodname']).args()

        params = req_doc.getElementsByTagName('param')

        for current_param in range(0, params.length):

            node = params.item(current_param)

            value_node = node.getElementsByTagName('value').item(0)

            value = self.get_param_value(value_node)

            current_arg = args[current_param]

            req_dict['args'][current_arg['name']] = value

        return req_dict

class XMLRPCResponseHandler(BaseResponseHandler):

    _content_type = 'text/xml'
    _stringify_res_dict = False
    datetime_re = None

    def get_xml_value(self, value, resp_doc):

        value_el = resp_doc.createElement('value')

        if isinstance(value, (str, unicode)):

            # Check for special cases base64, dateTime.iso8601

            if is_binary(value):

                base64_el = resp_doc.createElement('base64')
                base64_el.appendChild(
                    resp_doc.createTextNode(base64.b64encode(str(value)))
                )

                value_el.appendChild(base64_el)

            elif self.datetime_re.match(value):

                datetime_el = resp_doc.createElement('dateTime.iso8601')
                datetime_el.appendChild(
                    resp_doc.createTextNode(value)
                )

                value_el.appendChild(datetime_el)

            elif isinstance(value, unicode):

                string_el = resp_doc.createElement('string')
                string_el.appendChild(
                    resp_doc.createTextNode(value)
                )

                value_el.appendChild(string_el)

            else:

                string_el = resp_doc.createElement('string')
                string_el.appendChild(
                    resp_doc.createTextNode(u(value))
                )

                value_el.appendChild(string_el)

        elif isinstance(value, int):

            int_el = resp_doc.createElement('int')
            int_el.appendChild(
                resp_doc.createTextNode(value)
            )

            value_el.appendChild(int_el)

        elif isinstance(value, float):

            double_el = resp_doc.createElement('double')
            double_el.appendChild(
                resp_doc.createTextNode(value)
            )

            value_el.appendChild(double_el)

        elif isinstance(value, bool):

            if value:
                value = 1
            else:
                value = 0

            boolean_el = resp_doc.createElement('boolean')
            boolean_el.appendChild(
                resp_doc.createTextNode(value)
            )

            value_el.appendChild(boolean_el)

        elif isinstance(value, datetime.time):

            datetime_el = resp_doc.createElement('dateTime.iso8601')
            datetime_el.appendChild(
                resp_doc.createTextNode(
                    value.strftime('%Y%m%dT%H:%M:%S')
                )
            )

            value_el.appendChild(datetime_el)

        elif isinstance(value, dict):

            struct_el = resp_doc.createElement('struct')

            for current_member in range(0, len(value.keys())):

                member_key = value.keys()[current_member]

                member_value_el = self.get_xml_value(
                    value[member_key],
                    resp_doc
                )

                member_el = resp_doc.createElement('member')

                name_el = resp_doc.createElement('name')
                name_el.appendChild(
                    resp_doc.createTextNode(member_key)
                )

                member_el.appendChild(name_el)
                member_el.appendChild(member_value_el)

                struct_el.appendChild(member_el)

            value_el.appendChild(struct_el)

        elif isinstance(value, list):

            array_el = resp_doc.createElement('array')
            data_el = resp_doc.createElement('data')

            for current_value in range(0, len(value)):
                data_value_el = self.get_xml_value(
                    value[current_value],
                    resp_doc
                )

                data_el.appendChild(data_value_el)

            array_el.appendChild(data_el)

            value_el.appendChild(array_el)

        return value_el

    def build_response(self,res_dict,sinfo,encoding):
        self.datetime_re = re.compile('\d{8}T\d{2}:\d{2}:\d{2}')

        value = res_dict['result']

        impl = getDOMImplementation()

        resp_doc = impl.createDocument(None, 'methodResponse', None)
        params_el = resp_doc.createElement('params')
        param_el = resp_doc.createElement('param')

        value_el = self.get_xml_value(value, resp_doc)

        param_el.appendChild(value_el)

        params_el.appendChild(param_el)

        resp_doc.documentElement.appendChild(params_el)

        return resp_doc.toxml(encoding = encoding)

class XMLRPCFaultHandler(BaseFaultHandler):
    _content_type = 'text/xml'
    _stringify_res_dict = False

    def build_fault_response(self,service_exc,sinfo,methodname,encoding,reflection):
        if service_exc.detail:
            detail = service_exc.detail
        else:
            detail = traceback.format_exc()
        if service_exc.hint:
            detail += "\n" + service_exc.hint

        detail = detail.replace('\r\n','\n')

        impl = getDOMImplementation()

        resp_doc = impl.createDocument(None, 'methodResponse', None)

        fault_el = resp_doc.createElement('fault')
        value_el = resp_doc.createElement('value')
        struct_el = resp_doc.createElement('struct')

        # Fault-Code

        code_member_el = resp_doc.createElement('member')

        code_name_el = resp_doc.createElement('name')
        code_name_el.appendChild(
            resp_doc.createTextNode('faultCode')
        )

        code_member_el.appendChild(code_name_el)

        code_value_el = resp_doc.createElement('value')

        code_value_int_el = resp_doc.createElement('int')
        code_value_int_el.appendChild(
            resp_doc.createTextNode('99')
        )

        code_value_el.appendChild(code_value_int_el)

        code_member_el.appendChild(code_value_el)

        struct_el.appendChild(code_member_el)

        # Fault-String

        string_member_el = resp_doc.createElement('member')

        string_name_el = resp_doc.createElement('name')
        string_name_el.appendChild(
            resp_doc.createTextNode('faultString')
        )

        string_member_el.appendChild(string_name_el)

        string_value_el = resp_doc.createElement('value')

        string_value_string_el = resp_doc.createElement('string')
        string_value_string_el.appendChild(
            resp_doc.createTextNode(service_exc.faultstring + "\n" + detail)
        )

        string_value_el.appendChild(string_value_string_el)

        string_member_el.appendChild(string_value_el)

        struct_el.appendChild(string_member_el)

        value_el.appendChild(struct_el)
        fault_el.appendChild(value_el)

        resp_doc.documentElement.appendChild(fault_el)

        return resp_doc.toxml(encoding = encoding)

@expose
class XMLRPCInterface(BaseInterface):

    def __init__(self,sinfo,**kw):
        def_kw = {
        'service_descriptor': XMLRPCServiceDescriptor,
        'request_handler': XMLRPCRequestHandler,
        'response_handler': XMLRPCResponseHandler,
        'fault_handler': XMLRPCFaultHandler}
        def_kw.update(kw)
        BaseInterface.__init__(self,sinfo,**def_kw)

    @staticmethod
    def _interface_name():
        return 'xmlrpc'

    @staticmethod
    def _accept_basetype(typ):
        return pytype_support.count(typ)>0

    @staticmethod
    def _accept_list():
        return True

    @staticmethod
    def _accept_dict():
        return True


# -*- coding: utf-8 -*-

from ladon.clients.jsonwsp import JSONWSPClient
import pprint,os
from os.path import join,dirname,abspath

base_url = 'http://localhost:8080'

files_dir = join(dirname(abspath(__file__)),'files')
download_dir = join(dirname(abspath(__file__)),'download')

for d in [files_dir,download_dir]:
  if not os.path.exists(d):
    os.mkdir(d)

def print_result(jsonwsp_resp):
	if jsonwsp_resp.status == 200:
		if 'result' in jsonwsp_resp.response_dict:
			pprint.pprint(jsonwsp_resp.response_dict['result'],indent=2)
		else:
			pprint.pprint(jsonwsp_resp.response_dict)
	else:
		print("A problem occured while communicating with the service:\n")
		print(jsonwsp_resp.response_body)

def testCalculator():

	global base_url
	print("\n\nTesting Calculator:\n")
	# Load the Calculator description
	
	calc_client = JSONWSPClient(base_url + '/Calculator/jsonwsp/description')

	# Add the numbers 23 and 45
	jsonwsp_resp = calc_client.add(a=23,b=45)
	print_result(jsonwsp_resp)


def testAlbumService():

	print("\n\nTesting AlbumService:\n")
	# Load the AlbumService description
	album_client = JSONWSPClient(base_url + '/AlbumService/jsonwsp/description')

	# Fetch albums containing the substring "Zoo" in the album title
	jsonwsp_resp = album_client.listAlbums(search_frase='Bowie')
	print_result(jsonwsp_resp)

	# Fetch all bands containing the substring "Bowie" in the band name
	jsonwsp_resp = album_client.listBands(search_frase='Bowie')
	print_result(jsonwsp_resp)


def testTransferService():
	global files_dir, download_dir

	print("\n\nTesting TransferService:\n")
	# Load the TransferService description
	transfer_client = JSONWSPClient(base_url + '/TransferService/jsonwsp/description')

	# File list for the upload() call
	file_list = []
	# list of file names fot the download() call
	name_list = []
	
	# Get a list of files in the "files" folder
	files = os.listdir(files_dir)
	for f in files:
		fpath = os.path.join(files_dir,f)
		# Check if the entry is a file
		if os.path.isfile(fpath):
			# Add the file as a TransferService.File object (for the upload() call)
			file_list += [ {
				'data': open(fpath,'rb'), # Attach the file using an open file-handle
				'name': f                 # The file name
			} ]
			
			# Add the file name to the list of file names (for the download() call)
			name_list += [f]
	
	# Upload multiple files (all files found in the "files" directory) in one request
	jsonwsp_resp = transfer_client.upload(incomming=file_list)
	print_result(jsonwsp_resp)

	# Download all the files we just uploaded in one request
	jsonwsp_resp = transfer_client.download(names=name_list)
	print_result(jsonwsp_resp)

	# The attachments are referenced as open file-handles in the response object
	# read their content and save it as files in the "download" folder.
	if jsonwsp_resp.status==200:
		for f in jsonwsp_resp.response_dict['result']:
			fp = open(os.path.join(download_dir,f['name']),'wb')
			fp.write(f['data'].read())
			fp.close()


if __name__=='__main__':
	testCalculator()
	testAlbumService()
	testTransferService()



from threading import Lock,Thread
import traceback,sys,time
import ladon.tools.log as log

def get_traceback():
	strio = StringIO()
	traceback.print_exc(file=strio)
	return strio.getvalue()

thread_id_locker = Lock()
common_lock = Lock()

thread_dict = {
	'running': [],
	'done': [],
	'thread_info': {}
}

thread_id = 0

def next_thread_id():
	global thread_id,thread_id_locker
	thread_id_locker.acquire()
	thread_id += 1
	thread_id_locker.release()
	return thread_id

def update_thread_dict(thread_id,running):
	global thread_dict,common_lock
	common_lock.acquire()
	if running:
		if thread_id not in thread_dict['running']:
			thread_dict['running'] += [thread_id]
			try:
				thread_dict['done'].remove(thread_id)
			except ValueError as e:
				pass
	else:
		if thread_id not in thread_dict['done']:
			thread_dict['done'] += [thread_id]
			try:
				thread_dict['running'].remove(thread_id)
			except ValueError as e:
				pass
		
	common_lock.release()

def thread_count(running):
	global thread_dict,common_lock
	common_lock.acquire()
	tcnt = len(thread_dict['running' if running else 'done'])
	common_lock.release()
	return tcnt

def thread_running(thread_id):
	global thread_dict,common_lock
	common_lock.acquire()
	running = thread_id in thread_dict['running']
	common_lock.release()
	return running

def set_thread_info(thread_id,key,value):
	global thread_dict,common_lock
	common_lock.acquire()
	if thread_id not in thread_dict['thread_info']:
		thread_dict['thread_info'][thread_id] = {}
	thread_dict['thread_info'][thread_id][key] = value
	common_lock.release()

def get_thread_info(thread_id,key):
	global thread_dict,common_lock
	common_lock.acquire()
	val = thread_dict['thread_info'].get(thread_id,{}).get(key,None)
	common_lock.release()
	return val

def listappend_thread_info(thread_id,key,value):
	global thread_dict,common_lock
	common_lock.acquire()
	if thread_id not in thread_dict['thread_info']:
		thread_dict['thread_info'][thread_id] = {}
	l = thread_dict['thread_info'][thread_id].get(key,None)
	if not l:
		thread_dict['thread_info'][thread_id][key] = []
	thread_dict['thread_info'][thread_id][key].append(value)
	common_lock.release()

def setitem_thread_info(thread_id,key,dictkey,value):
	global thread_dict,common_lock
	common_lock.acquire()
	if thread_id not in thread_dict['thread_info']:
		thread_dict['thread_info'][thread_id] = {}
	l = thread_dict['thread_info'][thread_id].get(key,None)
	if not l:
		thread_dict['thread_info'][thread_id][key] = {}
	thread_dict['thread_info'][thread_id][key][dictkey] = value
	common_lock.release()

def thread_info(thread_id,key):
	global thread_dict,common_lock
	common_lock.acquire()
	val = thread_dict['thread_info'].get(thread_id,None)
	if not val==None:
		val = val.get(key,None)
	common_lock.release()
	return val

def all_thread_info_keys():
	global thread_dict,common_lock
	common_lock.acquire()
	keys = list(thread_dict['thread_info'])
	common_lock.release()	
	return keys

class ManagedThread(Thread):
	
	def post_event(self,identifier,*args,**kw):
		if self.event_cb:
			self.event_cb(identifier,*args,**kw)

	def __init__(self):
		self.thread_id = next_thread_id()
		if not getattr(self,'event_cb',None):
			self.event_cb = None
		super(ManagedThread,self).__init__()
	
	def run_managed(self):
		raise NotImplementedError
	
	def run(self):
		update_thread_dict(self.thread_id,True)
		self.update_progress(0.0)
		self.set_thread_info('start',time.time())
		try:
			self.run_managed()
		except:
			update_thread_dict(self.thread_id,False)
			log.write("Thread ID: %d (%s) failed with the following exception:\n%s" % (self.thread_id,self.__class__,log.get_traceback()))
		self.set_thread_info('stop',time.time())
		self.update_progress(1.0)
		update_thread_dict(self.thread_id,False)
	
	def update_progress(self,progress):
		if type(progress)!=float:
			return
		if progress>1:
			progress = 1
		if progress<0:
			progress = 0
		self.set_thread_info('progress',progress)
	
	def progress(self):
		self.get_thread_info('progress',progress)
		
	def set_thread_info(self,key,val):
		set_thread_info(self.thread_id,key,val)

	def get_thread_info(self,key):
		return get_thread_info(self.thread_id,key)

	def listappend_thread_info(self,key,val):
		return listappend_thread_info(self.thread_id,key,val)
	
	def setitem_thread_info(self,key,dictkey,val):
		return setitem_thread_info(self.thread_id,key,dictkey,val)

	def thread_info(self,key):
		return thread_info(self.thread_id,key)
 



# -*- coding: utf-8 -*-
from ladon.ladonizer import ladonize
from ladon.types.ladontype import LadonType
from ladon.types.attachment import attachment
from ladon.compat import PORTABLE_STRING
from os.path import dirname,abspath,join
import os

upload_dir = join(dirname(abspath(__file__)),'upload')

class File(LadonType):
	data = attachment
	name = PORTABLE_STRING

class TransferService(object):
	"""
	TransferService demonstrates how easy it is to write a service that can transport
	files forth and back between server and clients.
	"""

	@ladonize([File],rtype=int)
	def upload(self,incomming):
		"""
		Upload multiple files at once. Files are stored in the folder services/upload.
		
		@param incomming: A list of File objects containing file data and name
		@rtype: Returns 1 on success
		"""
		global upload_dir
		if not os.path.exists(upload_dir):
			os.mkdir(upload_dir)
		for upload_item in incomming:
			f = open(join(upload_dir,upload_item.name),'wb')
			f.write(upload_item.data.read())
			f.close()
		return 1

	@ladonize([PORTABLE_STRING], rtype=[File])
	def download(self,names):
		"""
		@publisher: docutils
		
		- Test
		- Test 2
		
		+------------+------------+-----------+ 
		| Header 1   | Header 2   | Header 3  | 
		+============+============+===========+ 
		| body row 1 | column 2   | column 3  | 
		+------------+------------+-----------+ 
		| body row 2 | Cells may span columns.| 
		+------------+------------+-----------+ 
		| body row 3 | Cells may  | - Cells   | 
		+------------+ span rows. | - contain | 
		| body row 4 |            | - blocks. | 
		+------------+------------+-----------+
		
		Kode eksempel::
			
			def test(self):
				print "oijfwe"
		
		

		Download multiple files at once. For each name in the <b>names</b> the service
		attempts to find a file in service/upload that matches it. If a name does not
		have a matching file it is ignored.
		
		@param names: A list of the file names
		@rtype: Returns a list of File objects
		"""
		global upload_dir
		response = []
		for name in names:
			f = File()
			f.name = name
			f.data = attachment(open(join(upload_dir,name),'rb'))
			response += [f]
		return response

#from ladon.ladonizer.collection import global_lsc
#print(global_lsc.services_by_name("TransferService")[0].method("upload"))



# -*- coding: utf-8 -*-

from ladon.interfaces.base import BaseInterface,ServiceDescriptor,BaseRequestHandler,BaseResponseHandler,BaseFaultHandler
from ladon.interfaces import expose
from ladon.compat import type_to_jsontype,pytype_support,PORTABLE_STRING
import json,sys,traceback
from ladon.exceptions.service import ServiceFault
from ladon.exceptions.base import LadonException

def _add_passback_params(res_dict,passback_dict):
	merged_dict = dict((k,v) for (k,v) in res_dict.items())	
	for k,v in passback_dict.items():
		merged_dict[k] = v
	return merged_dict

class RequestPropFault(ServiceFault):
	def __init__(self,prop, passback_dict):
		self.prop = prop
		self.passback_dict = passback_dict
		super(RequestPropFault,self).__init__('service','Request doesn\'t have "%s" property.' % self.prop, None, 2)

	def __str__(self):
		return self.faultstring

class RequestParamFault(ServiceFault):
	def __init__(self,param, passback_dict):
		self.param = param
		self.passback_dict = passback_dict
		super(RequestParamFault,self).__init__('service','Request doesn\'t have "%s" parameter.' % self.param, None, 2)

	def __str__(self):
		return self.faultstring
	
class MethodArgsCountFault(ServiceFault):
	def __init__(self,methodname,targsscount,gargscount,passback_dict):
		self.methodname = methodname
		self.targsscount = targsscount
		self.gargscount = gargscount
		self.passback_dict = passback_dict
		super(MethodArgsCountFault,self).__init__('service','Method "%s" takes exactly %s arguments, %s given.' % \
			(self.methodname,self.targsscount,self.gargscount), None, 2)

	def __str__(self):
		return self.faultstring
	
class RequestParamsArrayFault(ServiceFault):
	def __init__(self,passback_dict):
		self.passback_dict = passback_dict
		super(RequestParamsArrayFault,self).__init__('service','Params must be array of objects.',None,2)

	def __str__(self):
		return self.faultstring

class JSONRPCServiceDescriptor(ServiceDescriptor):	
	javascript_type_map = type_to_jsontype
	version = '1.0'
	_content_type = 'application/json'

	def generate(self,servicename,servicenumber,typemanager,methodlist,service_url,encoding,**kw):
		type_dict = typemanager.type_dict
		type_order = typemanager.type_order

		def map_type(typ):
			if typ in JSONRPCServiceDescriptor.javascript_type_map:
				return JSONRPCServiceDescriptor.javascript_type_map[typ]
			else:
				return typ.__name__

		desc = {
			'servicename': servicename,
			'url': service_url,
			'type': 'jsonrpc/description',
			'version': self.version,
			'types': {},
			'methods': {}
		}
		
		types = desc['types']
		for typ in type_order:
			if type(typ)==dict:
				desc_type = {}
				types[typ['name']] = desc_type
				for k,v,props in typ['attributes']:
					if type(v)==list:
						desc_type_val = [map_type(v[0])]
					else:
						desc_type_val = map_type(v)
					desc_type[k] = desc_type_val

		methods = desc['methods']
		for m in methodlist:
			desc_mparams = {}
			order = 1
			desc_method = {'params': desc_mparams, 'doc_lines': m._method_doc}
			methods[m.name()] = desc_method
			for arg in m.args():
				if [list,tuple].count(type(arg['type'])):
					desc_param_type = [map_type(arg['type'][0])]
				else:
					desc_param_type = map_type(arg['type'])
				desc_mparams[arg['name']] = {
					"type": desc_param_type,
					"def_order": order,
					"optional": arg['optional'],
					}
				if 'doc' in arg:
					desc_mparams[arg['name']]["doc_lines"] = arg['doc']
				else:
					desc_mparams[arg['name']]["doc_lines"] = []
				order += 1
				
			if [list,tuple].count(type(m._rtype)):
				desc_rtype = [map_type(m._rtype[0])]
			else:
				desc_rtype = map_type(m._rtype)
			desc_method['ret_info'] = {
				'type': desc_rtype,
				'doc_lines': m._rtype_doc
			}
			
		if sys.version_info[0]>=3:
			return json.dumps(desc)
		return json.dumps(desc,encoding=encoding)

class JSONRPCRequestHandler(BaseRequestHandler):
	def parse_request(self,json_body,sinfo,encoding):
		def parse_number(x):
			return PORTABLE_STRING(x)
		def parse_constant(x):
			if x=='null':
				return PORTABLE_STRING("None")
			return PORTABLE_STRING(x)
		req_dict = json.loads(PORTABLE_STRING(json_body,encoding), parse_int=parse_number, parse_float=parse_number, \
			parse_constant=parse_constant)
		passback_dict = self.get_passback_params(req_dict)
		if 'method' not in req_dict:
			raise RequestPropFault('method',passback_dict)
		if 'params' not in req_dict:
			raise RequestPropFault('params',passback_dict)
		if 'id' not in req_dict:
			raise RequestPropFault('id',passback_dict)
		minfo = sinfo.methods[req_dict['method']]
		params = req_dict['params']
		if params is not None and type(params) is not list:
			raise RequestParamsArrayFault(passback_dict)
		params_len = len(params) if params is not None else 0
		args_len = len(minfo.args())
		if params_len == 0 and  args_len> 0:
			raise RequestParamFault(minfo.args()[0]['name'],passback_dict)
		elif params_len < args_len:
			raise RequestParamFault(minfo.args()[params_len]['name'],passback_dict)
		elif params_len > args_len:
			raise MethodArgsCountFault(req_dict['method'], args_len, params_len,passback_dict)
		req_dict['args'] = {}
		if params is not None:
			for i in range(len(params)):
				req_dict['args'][minfo.args()[i]['name']] = params[i]
		req_dict['methodname'] = req_dict['method']
		del req_dict['params']
		del req_dict['method']
		return req_dict
	
	def get_passback_params(self, req_dict):
		if 'id' in req_dict:
			return {'id': req_dict['id']}
		else:
			return {}

class JSONRPCResponseHandler(BaseResponseHandler):
	_content_type = 'application/json'
	_stringify_res_dict = False
	
	def build_response(self,res_dict,sinfo,encoding):
		res_dict['error'] = None
		del res_dict['servicenumber']
		del res_dict['servicename']
		del res_dict['method']
		return json.dumps(res_dict,ensure_ascii=False).encode(encoding)

class JSONRPCFaultHandler(BaseFaultHandler):
	_content_type = 'application/json'
	_stringify_res_dict = False
	
	def build_fault_response(self,service_exc,sinfo,methodname,encoding,reflection):
		if service_exc.detail:
			detail = service_exc.detail
		else:
			detail = traceback.format_exc()
		detail = detail.replace('\r\n','\n').split('\n')
		fault_dict = {
			'result': None,
			'error': {
				'code': service_exc.faultcode,
				'string': service_exc.faultstring,
				'detail': detail,
				'filename': service_exc.mod,
				'lineno': service_exc.lineno,
				'hint': service_exc.hint
			},
		}
		if hasattr(service_exc,'passback_dict'):
			fault_dict = _add_passback_params(fault_dict,service_exc.passback_dict)
		return json.dumps(fault_dict,ensure_ascii=False).encode(encoding)

@expose
class JSONRPCInterface(BaseInterface):
	def __init__(self,sinfo,**kw):
		def_kw = {
			'service_descriptor': JSONRPCServiceDescriptor,
			'request_handler': JSONRPCRequestHandler,
			'response_handler': JSONRPCResponseHandler,
			'fault_handler': JSONRPCFaultHandler}
		def_kw.update(kw)
		BaseInterface.__init__(self,sinfo,**def_kw)

	@staticmethod
	def _interface_name():
		return 'jsonrpc10'

	@staticmethod
	def _accept_basetype(typ):
		return pytype_support.count(typ)>0

	@staticmethod
	def _accept_list():
		return True

	@staticmethod
	def _accept_dict():
		return False


import os

_version_info = {
	'major': 0,
	'minor': 9,
	'micro': 30
}

__version__ = "%(major)s.%(minor)s.%(micro)s" % _version_info

def package_root(*subpath):
	root_parts = [__file__,'..']
	if subpath:
		root_parts += list(subpath)
	return os.path.abspath(os.path.join(*root_parts))



# -*- coding: utf-8 -*-

import inspect

class BaseInterface(object):
	"""All interface implementations must descend from BaseInterface. The interface
	implementation can be thought of as an aggregator that ties the three basic functions
	of a web service protocol together:
	
		1. ServiceDescriptor - A generator class that can provide a description for the service.
		   ie. WSDL for soap.
		2. BaseRequestHandler - A handler that can parse the raw request objects from the client and
		   convert them into a so-called req_dict (Request dictionary)
		3. BaseResponseHandler - A handler that can process the res_dict (Result Dictionary) of a
		   service call and build a response object that fit's the protocol being implemented.
		4. BaseFaultHandler - A handler that can convert a ladon.exceptions.service.ServiceFault object
		   to a fault response that fits the interface.
	
	"""
	def __init__(self,sinfo,**kw):
		self._sinfo = sinfo
		sd = ServiceDescriptor()
		if 'service_descriptor' in kw:
			sd = kw['service_descriptor']
			if inspect.getmro(sd).count(ServiceDescriptor):
				self._service_descriptor = sd()
		if 'request_handler' in kw:
			req_handler = kw['request_handler']
			if inspect.getmro(req_handler).count(BaseRequestHandler):
				self._request_handler = req_handler()
		if 'response_handler' in kw:
			resp_handler = kw['response_handler']
			if inspect.getmro(resp_handler).count(BaseResponseHandler):
				self._response_handler = resp_handler()
		if 'fault_handler' in kw:
			fault_handler = kw['fault_handler']
			if inspect.getmro(fault_handler).count(BaseFaultHandler):
				self._fault_handler = fault_handler()

	@staticmethod
	def _interface_name(typ):
		return None

	@staticmethod
	def _accept_basetype(typ):
		return False

	@staticmethod
	def _accept_list():
		return False

	@staticmethod
	def _accept_dict():
		return False
	
	def service_info(self):
		return self._sinfo
	
	def parse_request(self,soap_body,encoding='UTF-8'):
		return self._request_handler.parse_request(soap_body,self._sinfo,encoding)

	def build_response(self,res_dict,encoding='UTF-8'):
		return self._response_handler.build_response(res_dict,self._sinfo,encoding)

	def build_fault_response(self,exc,methodname=None,encoding='UTF-8',reflection=None):
		return self._fault_handler.build_fault_response(exc,self._sinfo,methodname,encoding,reflection)

	def stringify_res_dict(self):
		return self._response_handler._stringify_res_dict

	def response_content_type(self):
		return self._response_handler._content_type

	def description(self,service_url,encoding='UTF-8',**kw):
		return self._service_descriptor.generate(
			self._sinfo.servicename,
			self._sinfo.servicenumber,
			self._sinfo.typemanager,
			self._sinfo.method_list(),
			service_url,encoding,
			**kw)

	def description_content_type(self):
		return self._service_descriptor._content_type

class ServiceDescriptor(object):
	
	_content_type = 'text/plain'
	
	def generate(self,servicename,servicenumber,typemanager,methodlist,encoding,**kw):
		"""
		Implement the method that can generate a service description file for the
		type of interface you are developing. Thus if you were developing a new
		and better SOAP interface and a new and better service descriptor for it,
		you will be implementing a WSDL generator in this method.
		
		@param servicename This is a string containing the service class name
		@param servicenumber An integer that reflects the order in which the service classes are parsed (useful for creating namespaces)
		@param typemanager An instance of ladon.ladonizer.collection.TypeManager which manages the types used by the service
		@param methodlist The list of methods exposed by the service
		@param encoding The client-side controlled encoding to use in the descriptor
		
		@rtype String representation of the service descriptor (ie. wsdl-formatted if it were SOAP)
		"""
		return ''


class BaseRequestHandler(object):
	
	def parse_request(self,req,sinfo,encoding):
		return {}


class BaseResponseHandler(object):
	
	"""
	This is the base class of all response handlers. To implement a new response handler
	inherit it and overload the build_response() method so it implements the protocol you are
	extending Ladon with.
	
	set _stringify_res_dict to True if you want the Ladon dispatcher to convert all result
	values to unicode regardless of their defined types during ladonization. This can be useful
	in some interfaces. ie. SOAP where all values are presented the same way inside XML tags no
	matter if it is a string or a number (example: <name>string</name><age>37</age>)
	"""
	_content_type = 'text/plain'
	_stringify_res_dict = False
	
	def build_response(self,method_result,sinfo,encoding):
		return ''

class BaseFaultHandler(object):
	
	"""
	This is the base class of all fault handlers. To implement a new fault handler
	inherit it and overload the build_fault_response() method so it implements the
	protocol you are extending Ladon with.
	
	set _stringify_res_dict to True if you want the Ladon dispatcher to convert all result
	values to unicode regardless of their defined types during ladonization. This can be useful
	in some interfaces. ie. SOAP where all values are presented the same way inside XML tags no
	matter if it is a string or a number (example: <name>string</name><age>37</age>)
	"""
	_content_type = 'text/plain'
	_stringify_res_dict = False
	
	def build_fault_response(self,exc,sinfo,methodname,encoding):
		return ''



# -*- coding: utf-8 -*-

# This module can parse and create multipart-messages. The MultiPartReader
# is the parser that can read a multipart-message and stream the content
# into temporary files on the local filesystem.
# MultiPartWriter can create a multipart-message from strings or files on
# the local filesystem.
#
# Both classes read data in chunks and write their results to the filesystem
# in a stream-like manner. This way they will never take up more memory than
# is setup using the chunk_size argument.

import re,tempfile,os,time,hashlib
from ladon.compat import PORTABLE_BYTES, PORTABLE_STRING

rx_2linefeeds = re.compile(b'\n\n|\r\n\r\n',re.M)
rx_headers = re.compile(b'^([-_a-zA-Z0-9]+): (.*)$',re.M)

def upperkey(pair):
	return (pair[0].upper().replace(b'-',b'_'),pair[1])

class MultiPartReader(object):
	def __init__(self,chunk_size,boundary,req,content_size=None):
		self.chunk_size = chunk_size
		self.content_size=content_size
		self.boundary = boundary
		rx_boundary = re.escape(boundary)
		self.rx_boundary_split = re.compile(b'--<b>\n|\n--<b>\n|\n--<b>--|--<b>\r\n|\r\n--<b>\r\n|\r\n--<b>--'.replace(b'<b>',boundary.replace('"','')) ,re.M)
		self.len_rest_chunk = len(boundary)+16
		self.rest_chunk = b''
		self.data = b''
		self.req = req
		self.attachments = {}
		self.attachments_by_id = {}
		self.eos = False
		self.interface_request = b''
		self.part_count = 0
		self.attachment_headers = b''
		self.attachment_headers_parsed = False
		self.tmpdir = None
		self.fd = None
		self.request_bytes_read = 0
		self.attachment_bytes_size = 0
		self.raw_dump_rest = False
		self.request_bytes_read = 0
		self.attachment_count = 0
		self.attachment_bytes_size = 0
		
	def write(self,data,eop=False):
		"""
		@param eop: End of parts
		"""
		global rx_2linefeeds,rx_headers
		if self.attachment_headers_parsed == False:
			self.attachment_headers += data
			m = rx_2linefeeds.search(self.attachment_headers)
			if m:
				data = self.attachment_headers[m.end():]
				self.attachment_headers = self.attachment_headers[:m.start()]
				self.attachment_headers_parsed = True
				
		if self.attachment_headers_parsed == True:
			self.attachment_bytes_size += len(data)
			if self.part_count==0:
				self.interface_request += data
				self.interface_request_headers = dict(map(upperkey,rx_headers.findall(self.attachment_headers)))
			else:
				os.write(self.fd,data)
			
			if eop==True:
				if self.fd != None:
					os.close(self.fd)
					headers_dict = dict(map(upperkey,rx_headers.findall(self.attachment_headers)))
					self.attachments[self.part_count-1]['size'] = self.attachment_bytes_size
					self.attachments[self.part_count-1]['headers'] = headers_dict
					if b'CONTENT_ID' in headers_dict:
						self.attachments_by_id[headers_dict[b'CONTENT_ID']] = self.attachments[self.part_count-1]
				
				if not self.eos:
					self.attachments[self.part_count] = {}
					self.fd,self.attachments[self.part_count]['path'] = tempfile.mkstemp('','content_',self.tmpdir)
					self.part_count += 1
				self.attachment_bytes_size = 0
				self.attachment_headers = b''
				self.attachment_headers_parsed = False
		
	def read_chunk(self):
		read_to_end = False
		read_size = self.chunk_size
		if self.content_size and self.content_size < self.request_bytes_read + read_size:
			read_size = self.content_size - self.request_bytes_read
		chunk = self.req.read(read_size)
		self.request_bytes_read += len(chunk)
		if chunk=='' or len(chunk)<self.chunk_size:
			read_to_end = True
		
		focus = self.rest_chunk+chunk
		parts = self.rx_boundary_split.split(focus)
		
		if len(parts)>1:
			self.data+=parts[0]
			self.write(self.data,True)
		
			for part in parts[1:-1]:
				self.write(part,True)
			self.data = b''
		if read_to_end == True:
			self.eos = True
		self.write(parts[-1:][0][:-self.len_rest_chunk])
		if self.eos:
			os.close(self.fd)
			os.unlink(self.attachments[self.part_count-1]['path'])
		self.rest_chunk = parts[-1:][0][-self.len_rest_chunk:]


class MultiPartWriter(object):
	def __init__(self,req,chunk_size=50000,boundary=None):
		self.chunk_size = chunk_size
		if not boundary:
			md5=hashlib.md5()
			md5.update(str(time.time()).encode('iso-8859-1'))
			boundary = md5.hexdigest().encode('iso-8859-1')
		self.boundary = boundary
		self.req = req
	
	def add_attachment(self,data,content_type,content_id,headers={}):
		"""
		Add an attachment. Data must be binary meaning str for Python 2,
		bytes for Python 3 or a file-like object opened in binary mode.
		Content-type, content-id and strings in the headers dict can be
		strings but if they cannot be encoded to iso-8859-1 an exception
		is raised.
		Why? Because the result of this method is a binary object that
		will ultimately be inserted in a multipart message stream which
		is pure binary.
		"""
		if type(content_type)==PORTABLE_STRING:
			content_type = content_type.encode('iso-8859-1')
		if type(content_id)==PORTABLE_STRING:
			content_id = content_id.encode('iso-8859-1')
		self.req.write(b'--'+self.boundary+b'\n')
		self.req.write(b'Content-Type: '+content_type+b'\n')
		self.req.write(b'Content-ID: '+content_id+b'\n')
		for h_name,h_value in headers.items():
			try:
				if type(h_name)==PORTABLE_STRING:
					h_name = h_name.encode('iso-8859-1')
				if type(h_value)==PORTABLE_STRING:
					h_value = h_value.encode('iso-8859-1')
				self.req.write(h_name+b': '+h_value+b'\n')
			except:
				# ignore headers that can not be iso-8859-1 encoded or
				# otherwise fail to be added to the message part
				print('Failed header')
				pass
		self.req.write(b'\n')
		if hasattr(data,'read'):
			while 1:
				chunk = data.read(self.chunk_size)
				self.req.write(chunk)
				if chunk=='' or len(chunk)<self.chunk_size:
					break
			self.req.write(b'\n')
		elif type(data)==PORTABLE_BYTES:
			self.req.write(data)
			self.req.write(b'\n')

	def done(self):
		self.req.write(b'--'+self.boundary+b'--')


class AttachmentHandler(object):
	def __init__(self):
		self.cid_seq = 0
		self.attachments_by_cid = {}
		self.fs_attachments_map_cid = {}
	
	def get_next_cid(self):
		self.cid_seq += 1
		return 'ladon-attachment-%d' % self.cid_seq
		
	
	def add_attachment(self,attachment):
		if hasattr(attachment.bufferobj,'name'):
			# Could be a filesystem based attachment
			if os.path.exists(attachment.bufferobj.name):
				# Exists on filesystem
				fs_abspath = os.path.abspath(attachment.bufferobj.name)
				if fs_abspath not in self.fs_attachments_map_cid:
					# First encounter - no redundancy
					cid = self.get_next_cid()
					self.fs_attachments_map_cid[fs_abspath] = cid
					self.attachments_by_cid[cid] = attachment
					return 'cid:%s' % cid
				else:
					return 'cid:%s' % self.fs_attachments_map_cid[fs_abspath]
		else:
			cid = self.get_next_cid()
			self.attachments_by_cid[cid] = attachment
			return 'cid:%s' % cid


if __name__=='__main__':
	f=open('test.json','rb')
	mh=MultiPartReader(20000,b'493f4e46b6690ac551db9d306e7c5458',f)
	mh.read_chunk()
	while not mh.eos:
		mh.read_chunk()
	print(str(mh.interface_request_headers))
	print(str(mh.attachments_by_id))


	#f = open('test.json','wb')
	#mpw = MultiPartWriter(f)

	#a1=open('json-req','rb')
	#mpw.addAttachment(a1,'application/json, charset=iso-8859-1','dwezvfqrweverg0')
	#a1.close()

	#a1=open('/home/jakob/description','rb')
	#mpw.addAttachment(a1,'text/plain','dwezvfqrweverg1')
	#a1.close()

	#a1=open('/home/jakob/Billeder/trundfri.png','rb')
	#mpw.addAttachment(a1,'image/png','dwezvfqrweverg2')
	#a1.close()

	#mpw.done()
	#f.close()



from ladon.compat import PORTABLE_STRING
from ladon.ladonizer import ladonize
import time

class TaskService(object):

	@ladonize(PORTABLE_STRING, int, rtype=int, tasktype=True)
	def taskExample(self,session_id,counter,**kw):
		set_progress = kw['update_progress']
		repeat = 60.0
		for i in range(int(repeat)):
			time.sleep(.5)
			set_progress(float(i+1.0)/repeat)
		return counter



# -*- coding: utf-8 -*-
from ladon.server.wsgi import LadonWSGIApplication
import wsgiref.simple_server
from os.path import normpath,abspath,dirname,join
from ladon.tools.log import set_loglevel,set_logfile,set_log_backup_count,set_log_maxsize
set_logfile(join(dirname(normpath(abspath(__file__))),'examples.log'))
set_loglevel(6) # debug
set_log_backup_count(50)
set_log_maxsize(50000)

scriptdir = dirname(abspath(__file__))
service_modules = ['calculator','albumservice','transferservice','shopservice','taskservice']

# Create the WSGI Application
application = LadonWSGIApplication(
	service_modules,
	[join(scriptdir,'services'),join(scriptdir,'appearance')],
	catalog_name = 'Ladon Service Examples',
	catalog_desc = 'The services in this catalog serve as examples to how Ladon is used')

if __name__=='__main__':
	# Starting the server from command-line will create a stand-alone server on port 8080
	port = 8080
	print("\nExample services are running on port %(port)s.\nView browsable API at http://localhost:%(port)s\n" % {'port':port})

	server = wsgiref.simple_server.make_server('', port , application)
	server.serve_forever()



# -*- coding: utf-8 -*-
import sys
from ladon.exceptions.types import *
from ladon.compat import PORTABLE_BYTES,PORTABLE_STRING,pytype_support,safe_conversions

class TypeConverter(object):
	
	"""The type converter mainly serves as a tool for the dispatcher providing the means to
	validate and convert between the interface and function layers
	"""
	
	def __init__(self,encoding='UTF-8',allow_unsafe_conversion=False,only_strings_to_unicode=True):
		"""In the TypeConverter constructor you can set conversion properties like which
		encoding to use when converting forth and back between unicode to str (or bytes and
		str for python 3). The default encoding is UTF-8.
		allow_unsafe_conversion is a boolean that tells the TypeConverter if unsafe conversions are allowed,
		An example of an unsafe conversion is float to int, as any floating point part will be truncated
		in the conversion.
		only_strings_to_unicode tells the converter to let types other than unicode and str (or bytes and
		str in python 3) be converted to the expected type when calling to_unicode_string
		"""
		# Converter options
		self.encoding=encoding
		self.allow_unsafe_conversion = allow_unsafe_conversion
		self.only_strings_to_unicode = only_strings_to_unicode

	def from_unicode_string(self,val,typ,prefilters=None,postfilters=None):
		"""Convert a unicode string to certain types.
		"""
		# Run incoming_raw filters aka. incoming prefilters
		if prefilters:
			for func in prefilters:
				val = func(val)
		
		vtyp = type(val)
		
		# The following fix (accepting booleans) has been added because
		# keyword parameter parse_constant of json.loads has changed to
		# not react on true/false (http://bugs.python.org/issue14692)
		if vtyp==bool and typ==bool:
			val = PORTABLE_STRING(val)
			vtyp = PORTABLE_STRING
		
		# Check the value type
		if not vtyp==PORTABLE_STRING:
			raise NonUnicodeError(type(val))
		
		# unicode to unicode or raw string
		if typ in [PORTABLE_BYTES,PORTABLE_STRING]:
			if typ==PORTABLE_BYTES:
				val = val.encode(self.encoding)
			# Run incoming filters aka. incoming postfilters
			if postfilters:
				for func in postfilters:
					val = func(val)
			return val
		
		# other primitive types
		try:
			if typ==bool and (val[0].upper()==PORTABLE_STRING('F') or val.strip()=='0'):
				val = False
			else:
				val = typ(val)
			# Run incoming filters aka. incoming postfilters
			if postfilters:
				for func in postfilters:
					val = func(val)
			return val
		except Exception as e:
			raise FromUnicodeConversionError(typ,str(e))


	def to_unicode_string(self,val,expected_typ,prefilters=None,postfilters=None):
		"""Convert a primitive type to unicode. The method uses TypeConverter
		options in the decition making, and encoding.
		"""
		# Run outgoing filters aka. outgoing prefilters
		if prefilters:
			for func in prefilters:
				val = func(val)
		typ = type(val)
		# Expecting unicode or raw string
		if expected_typ in [PORTABLE_BYTES,PORTABLE_STRING]:
			# No unsafe conversion can enter here as long as the user's service method obeys
			# the internal encoding it advertises for raw strings (aka bytes in python 3)
			if typ==PORTABLE_STRING:
				pass
			elif typ==PORTABLE_BYTES:
				val = PORTABLE_STRING(val,self.encoding)
			else:
				val = PORTABLE_STRING(val)
			# Run outgoing_raw filters aka. outgoing postfilters
			if postfilters:
				for func in postfilters:
					val = func(val)
			return val
		
		if not self.allow_unsafe_conversion:
			# Check if the conversion is unsafe
			if expected_typ not in safe_conversions[typ]:
				raise UnsafeConversionDisabled(typ,expected_typ)
		try:
			val = expected_typ(val)
		except Exception as e:
			raise UnsafeConversionError(typ,expected_typ,str(e))
		if self.only_strings_to_unicode:
			pass
		else:
			if expected_typ==bool:
				val = PORTABLE_STRING(val).lower()
			val = PORTABLE_STRING(val)
		# Run outgoing_raw filters aka. outgoing postfilters
		if postfilters:
			for func in postfilters:
				val = func(val)
		return val



# -*- coding: utf-8 -*-

catalog_default_css = '''
	body { background-color: white; margin: 0px; padding: 0px; font-size: 12px; font-family: "Lucida Console", Monaco, monospace; }
	div.catName { height: 22px; padding: 10px; background-color: #000; font-size: 20px; color: white; }
	div.catDesc { padding: 20px 20px 10px; }
	ul.catService { }
	ul.catService li { padding-bottom: 10px; }
	div.catInterfaces { padding: 0px 10px 10px 30px; }
	div.catGen { text-align: center; font-style: italic; margin-top: 20px; margin-bottom: 20px; } 
'''

service_default_css = '''
	body { background-color: white; margin: 0px; padding: 0px; font-size: 12px; font-family: "Lucida Console", Monaco, monospace; }
	div.serviceName { height: 22px; padding: 10px; background-color: #000; font-size: 20px; color: white; }
	div.title {position: relative; float:left}
	form.skin-selector {position:relative; float: right; font-size: 12pt;}
	div.serviceDesc { padding: 20px 20px 10px; }
	span.descriptionHead { font-weight: bold; }
	span.interfaceHead { padding-left: 20px; font-weight: bold; }
	span.methodsHead { padding-left: 20px; font-weight: bold; }
	span.typesHead { padding-left: 20px; font-weight: bold; }
	div.methodDef { font-size: 16px; font-family: courier }
	div.typeDef { font-size: 16px; font-family: courier }
	span.methodName { color: darkred; font-weight: bold; font-family: courier }
	span.typeName { color: darkred; font-weight: bold; font-family: courier }
	span.paramType { color: blue }
	span.paramName { color: black; font-weight: bold }
	li.paramDesc { padding-bottom: 10px }
	li.methodEntry { padding-bottom: 15px }
	li.typeAttrib { padding-bottom: 10px }
	p.docLines { margin-top: 4px; color: black; margin-bottom: 4px }
	ul.interfaceList { }
	ul.interfaceList li { padding-bottom: 10px; }
	div.catGen { text-align: center; font-style: italic; margin-top: 20px; }
'''



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys,json,types,pprint,os,tempfile,re
from optparse import OptionParser,OptionGroup
from ladon.tools.multiparthandler import MultiPartReader, MultiPartWriter
from ladon.types.attachment import attachment
from ladon.compat import PORTABLE_STRING,PORTABLE_STRING_TYPES

if sys.version_info[0]>=3:
	from urllib.parse import urlparse,splitport
	from http.client import HTTPConnection, HTTPSConnection
else:
	from urllib import splitport
	from urlparse import urlparse
	from httplib import HTTPConnection, HTTPSConnection

rx_ctype_charset = re.compile('charset\s*=\s*([-_.a-zA-Z0-9]+)',re.I)
rx_detect_multipart = re.compile('multipart/([^; ]+)',re.I)
rx_detect_boundary = re.compile('boundary=([^; ]+)',re.I)
rx_detect_attachment_ref = re.compile('^cid:(.+)$',re.I)

def probe_charset(headers,default='UTF-8'):
	global rx_ctype_charset
	try:
		res = rx_ctype_charset.findall(headers['CONTENT-TYPE'])
		if len(res):
			return res[0]
		return headers['HTTP-ACCEPT-CHARSET'].split(';')[0].split(',')[0]
	except:
		pass
	
	return default


def fix_attachment(val,attachment_map):
	if hasattr(val,'read'):
		while 1:
			cid = 'file%s' % attachment_map['cid_seq']
			attachment_map['cid_seq'] += 1
			if cid not in attachment_map['files']:
				break
		attachment_map['files'][cid] = val
		return 'cid:%s' % cid
		
	if type(val) in PORTABLE_STRING_TYPES and len(val)>0 and val[0]=='@' and os.path.isfile(val[1:]):
		cid = os.path.normpath(val[1:])
		if cid not in attachment_map:
			attachment_map['files'][cid] = open(cid,'rb')
			return 'cid:%s' % cid
	
	return None


def walk_args_dict(kw,attachment_map):
	for key,val in kw.items():
		if type(val)==tuple:
			val = list(val)
			kw[key] = val
		if type(val)==list:
			for vidx in range(len(val)):
				if type(val[vidx])==dict:
					walk_args_dict(val[vidx],attachment_map)
				else:
					attachment_ref = fix_attachment(val[vidx],attachment_map)
					if not attachment_ref == None:
						val[vidx] = attachment_ref
				
		elif type(val)==dict:
			walk_args_dict(val,attachment_map)
		else:
			attachment_ref = fix_attachment(val,attachment_map)
			if not attachment_ref == None:
				kw[key] = attachment_ref

class JSONWSPResponse(object):
	def __init__(self,status,reason,headers):
		self.response_body = ''
		self.response_dict = {}
		self.headers = headers
		self.status = status
		self.reason = reason
		self.attachments = {}
		self.dict_attachments = []


	def __del__(self):
		self.response_dict = {}
		for a in self.dict_attachments:
			a.bufferobj.close()
			del a

		for cid,a in self.attachments.items():
			fname = a.bufferobj.name
			a.bufferobj.close()
			del a
			os.unlink(fname)


	def check_attachment(self,val):
		global rx_detect_attachment_ref
		if type(val) == PORTABLE_STRING:
			m = rx_detect_attachment_ref.match(val)
			if m and m.groups()[0] in self.attachments:
				a = self.attachments[m.groups()[0]]
				attachment_copy = attachment(open(a.bufferobj.name,'rb'),a.size,a.headers,None)
				self.dict_attachments += [attachment_copy]
				return attachment_copy
		return None

	def walk_result_dict(self,result=None):
		if result==None:
			result = self.response_dict
		for key,val in result.items():
			if result==self.response_dict and key!='result':
				continue
			if type(val)==list:
				for vidx in range(len(val)):
					if type(val[vidx])==dict:
						self.walk_result_dict(val[vidx])
					else:
						attachment = self.check_attachment(val[vidx])
						if not attachment == None:
							val[vidx] = attachment
					
			elif type(val)==dict:
				self.walk_result_dict(val)
			else:
				attachment = self.check_attachment(val)
				if not attachment == None:
					result[key] = attachment


class JSONWSPClient(object):

	def __init__(self,description=None,url=None,debug=False,via_proxy=False,cookies={},headers= {}):
		
		self.description_loaded = False
		self.via_proxy = via_proxy
		self.cookie_dict = cookies
		self.header_dict = headers
		
		if description:
			self.parse_url(description)
			self.parse_description()
		if url:
			self.parse_url(url)
		self.debug = debug

	def parse_url(self,url):
		self.valid_url = True
		parseres = urlparse(url)
		self.scheme = parseres.scheme
		if self.scheme.lower()=="https":
			self.port = 443
		elif self.scheme.lower()=="http":
			self.port = 80
		else:
			self.valid_url = False
		self.hostname,custom_port = splitport(parseres.netloc)
		if str(custom_port).isdigit():
			self.port = int(custom_port)
		self.path = parseres.path

	def add_cookie(self,name,value):
		if self.cookie_dict == None:
			self.cookie_dict = {}
		self.cookie_dict[name] = value
	
	def remove_cookie(self,name):
		if name in self.cookie_dict:
			self.cookie_dict.pop(name)
			
	def add_header(self,name,value):
		if self.header_dict == None:
			self.header_dict = {}
		self.header_dict[name] = value
	
	def remove_header(self,name):
		if name in self.header_dict:
			self.header_dict.pop(name)
	
	def get_all_headers(self,extra_headers={}):
	
		headerlist = self.header_dict
		if not extra_headers == None:
			headerlist.update(extra_headers)
		
		cookieString = ""
		if not self.cookie_dict == None:
			for key in self.cookie_dict:
				if not cookieString == "":
					cookieString = cookieString + "; ";
				cookieString = cookieString + key+"="+self.cookie_dict[key]
		
		if "Cookie" in headerlist:
			headerlist["Cookie"] = headerlist["Cookie"]+"; "+cookieString
		else:
			headerlist["Cookie"] = cookieString
			
		return headerlist
		
	def call_method(self,method_name,**kw):
		attachment_map={'cid_seq':1,'files':{}}
		extra_headers = None
		if 'extra_headers' in kw:
			extra_headers = kw['extra_headers']
		walk_args_dict(kw,attachment_map)
		if self.description_loaded:
			minfo = self.method_info(method_name)
			mandatory_params = list(minfo['mandatory_params'])
			optional_params = list(minfo['optional_params'])
			for arg in kw.keys():
				if arg in mandatory_params:
					mandatory_params.remove(arg)
			if len(mandatory_params):
				return -1
		data = {
			'methodname': method_name
		}
		if 'mirror' in kw:
			data['mirror'] = kw.pop('mirror')	
		data['args'] = kw
		if len(attachment_map['files']):
			jsonwsp_response = self.post_mp_request(json.dumps(data),attachment_map['files'],extra_headers=extra_headers)
		else:
			jsonwsp_response = self.post_request(json.dumps(data),extra_headers=extra_headers)
		if jsonwsp_response.status==200:
			response_charset = probe_charset(jsonwsp_response.headers)
			jsonwsp_response.response_dict = json.loads(PORTABLE_STRING(jsonwsp_response.response_body,response_charset))
			jsonwsp_response.walk_result_dict()

		return jsonwsp_response


	def list_methods(self):
		return self.jsonwsp_description['methods'].keys()


	def method_info(self,method_name):
		minfo = self.jsonwsp_description['methods'][method_name]
		params = minfo['params']
		mandatory_params = []
		optional_params = []
		params_order = ['']*len(params)
		for pname,pinfo in params.items():
			params_order[pinfo['def_order']-1] = pname
		for pname in params_order:
			pinfo = params[pname]
			if pinfo['optional']:
				optional_params += [pname]
			else:
				mandatory_params += [pname]
			
		return {
			'method_name': method_name,
			'params_order': params_order,
			'mandatory_params': mandatory_params,
			'optional_params': optional_params,
			'params_info': params,
			'doc_lines': minfo['doc_lines'],
			'ret_info': minfo['ret_info']
		}


	def parse_description(self):
		jsonwsp_response = self.post_request("","",via_proxy=self.via_proxy)
		response_charset = probe_charset(jsonwsp_response.headers)
		self.jsonwsp_description = json.loads(PORTABLE_STRING(jsonwsp_response.response_body,response_charset))
		self.parse_url(self.jsonwsp_description['url'])
		for method_name in self.jsonwsp_description['methods'].keys():
			exec("def placeholder(self,**kw):\n\treturn self.call_method('%s',**kw)" % method_name)
			exec("self.%s = types.MethodType(placeholder,self)" % method_name)
		self.description_loaded = True


	def parse_response(self,response):
		headers = dict((k.upper(), v) for k, v in response.getheaders())
		jsonwsp_response = JSONWSPResponse(response.status, response.reason, headers)
		jsonwsp_charset = probe_charset(jsonwsp_response.headers)
		content_type = response.getheader('content-type')
		content_type = content_type.replace('\n','')
		multipart_match = rx_detect_multipart.findall(content_type)
		if len(multipart_match):
			multipart = multipart_match[0]
			boundary_match = rx_detect_boundary.findall(content_type)
			if len(boundary_match):
				boundary = boundary_match[0]
				mpr = MultiPartReader(20000,boundary.encode(jsonwsp_charset),response)
				mpr.read_chunk()
				while not mpr.eos:
					mpr.read_chunk()
				for cid,cinfo in mpr.attachments_by_id.items():
					jsonwsp_response.attachments[PORTABLE_STRING(cid,jsonwsp_charset)] = attachment(open(cinfo['path'],'rb'),cinfo['size'],cinfo['headers'],jsonwsp_charset)
				resdata = mpr.interface_request
				jsonwsp_response.response_body = resdata
		else:
			resdata = response.read()
			jsonwsp_response.response_body = resdata
		return jsonwsp_response


	def post_request(self,data,extra_path="",via_proxy=False,extra_headers=None):
		all_headers = self.get_all_headers(extra_headers)
		headers = {
			"Content-type": "application/json, charset=UTF-8",
			"Accept": "application/json,multipart/related" }
		if via_proxy:
			headers["Ladon-Proxy-Path"] = "%s://%s:%d%s" % (
				self.scheme.lower(),self.hostname,self.port,self.path)
		if self.scheme.lower()=='https':
			conn = HTTPSConnection(self.hostname,self.port)
		else:
			conn = HTTPConnection(self.hostname,self.port)
		req_path = self.path + '/' + extra_path
		if all_headers:
			headers.update(all_headers)
		if req_path.endswith('/description/'):
			print("GET")
			conn.request("GET", req_path, data, headers)
		else:
			conn.request("POST", req_path, data, headers)
		jsonwsp_response = self.parse_response(conn.getresponse())
		conn.close()
		return jsonwsp_response


	def post_mp_request(self,data,files,extra_path="",extra_headers={}):
		# Create the multipart request body
		buffer_fname = tempfile.mktemp()
		buffer_fp = open(buffer_fname,'wb')
		mpw = MultiPartWriter(buffer_fp)
		mpw.add_attachment(data.encode('utf-8'),'application/json, charset=UTF-8','body')
		for cid,fp in files.items():
			mpw.add_attachment(fp,'application/octet-stream',cid)
		mpw.done()
		buffer_fp.close()
		headers = {
			"Content-type": b'multipart/related; boundary=' + mpw.boundary,
			"Accept": b'application/json,multipart/related'}
		if self.scheme.lower()=='https':
			conn = HTTPSConnection(self.hostname,self.port)
		else:
			conn = HTTPConnection(self.hostname,self.port)
		req_path = self.path + '/' + extra_path
		buffer_fp = open(buffer_fname,'rb')
		
		all_headers = self.get_all_headers(extra_headers)
		
		if all_headers!=None:
			headers.update(all_headers)
		conn.request("POST", req_path, buffer_fp, headers)
		buffer_fp.close()
		jsonwsp_response = self.parse_response(conn.getresponse())
		conn.close()
		os.unlink(buffer_fname)
		return jsonwsp_response

if __name__=="__main__":
	service_url = sys.argv[1]
	method_name = sys.argv[2]
	cli = JSONWSPClient(service_url)
	if method_name in ['-h','--help']:
		print("\n  "+"\n  ".join(cli.list_methods()))
		sys.exit(0)
	minfo = cli.method_info(method_name)
	mparams = list(minfo['mandatory_params'])
	oparams = list(minfo['optional_params'])
	parser = OptionParser(usage="%s SERVICE_URL METHOD_NAME %s [OPTIONS]" % (sys.argv[0],' '.join(minfo['mandatory_params'])))
	mandatory_group = OptionGroup(parser, "Mandatory argument option-bindings",
		"All mandatory arguments to a service method can be "
		"given via normal arguments, but in some cases it "
		"is nessecary to have option-bindings to an argument "
		"like when using xargs and other commands.")
	optional_group = OptionGroup(parser, "Options","Options for %s" % method_name)
	for p in mparams:
		if minfo['params_info'][p]['type']=='boolean':
			mandatory_group.add_option(
				"--%s"%p.replace('_','-'), 
				dest=p, 
				action="store_true", 
				help='\n'.join(minfo['params_info'][p]['doc_lines']))
		else:
			mandatory_group.add_option(
				"--%s"%p.replace('_','-'),
				dest=p,
				help='\n'.join(minfo['params_info'][p]['doc_lines']))
	parser.add_option_group(mandatory_group)
	for p in oparams:
		if minfo['params_info'][p]['type']=='boolean':
			optional_group.add_option(
				"--%s"%p.replace('_','-'), 
				dest=p, 
				action="store_true", 
				help='\n'.join(minfo['params_info'][p]['doc_lines']))
		else:
			optional_group.add_option(
				"--%s"%p.replace('_','-'),
				dest=p,
				help='\n'.join(minfo['params_info'][p]['doc_lines']))
	parser.add_option_group(optional_group)
	(options, args) = parser.parse_args()
	
	method_call_args = {}

	for p in minfo['optional_params']:
		pval = eval("options.%s" % p)
		if pval != None:
			oparams.remove(p)
			method_call_args[p] = pval

	for p in minfo['mandatory_params']:
		pval = eval("options.%s" % p)
		if pval != None:
			mparams.remove(p)
			method_call_args[p] = pval
	
	if len(sys.argv)<3+len(mparams):
		print("Missing mandatory parameters: %s" % ' '.join(mparams))
		sys.exit(1)
	
	arg_idx = 0
	for arg in args[2:]:
		method_call_args[mparams[arg_idx]] = arg
		arg_idx += 1

	pprint.pprint(eval("cli.%s(**method_call_args)" % method_name).response_dict,indent=2)



######################## BEGIN LICENSE BLOCK ########################
# The Original Code is Mozilla Universal charset detector code.
# 
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 2001
# the Initial Developer. All Rights Reserved.
# 
# Contributor(s):
#   Mark Pilgrim - port to Python
#   Shy Shalom - original C code
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA
######################### END LICENSE BLOCK #########################

import constants, re

class CharSetProber:
    def __init__(self):
        pass
        
    def reset(self):
        self._mState = constants.eDetecting
    
    def get_charset_name(self):
        return None

    def feed(self, aBuf):
        pass

    def get_state(self):
        return self._mState

    def get_confidence(self):
        return 0.0

    def filter_high_bit_only(self, aBuf):
        aBuf = re.sub(r'([\x00-\x7F])+', ' ', aBuf)
        return aBuf
    
    def filter_without_english_letters(self, aBuf):
        aBuf = re.sub(r'([A-Za-z])+', ' ', aBuf)
        return aBuf
        
    def filter_with_english_letters(self, aBuf):
        # TODO
        return aBuf



from ladon.server.wsgi import LadonWSGIApplication
from os.path import normpath,abspath,dirname,join
from ladon.tools.log import set_loglevel,set_logfile,set_log_backup_count,set_log_maxsize
set_logfile(join(dirname(normpath(abspath(__file__))),'apache-examples.log'))
set_loglevel(6) # debug
set_log_backup_count(50)
set_log_maxsize(50000)

scriptdir = dirname(abspath(__file__))
service_modules = ['calculator','albumservice','transferservice','shopservice','taskservice']

# Create the WSGI Application
application = LadonWSGIApplication(
        service_modules,
        [join(scriptdir,'services'),join(scriptdir,'appearance')],
        catalog_name = 'Ladon Service Examples',
        catalog_desc = 'The services in this catalog serve as examples to how Ladon is used')



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import sys,os

if sys.version_info[0]==2:
	other_version = 3
else:
	other_version = 2

readme = changes = ''
if os.path.exists('README.rst'):
	readme = open('README.rst').read()
if os.path.exists('CHANGES.rst'):
	changes = open('CHANGES.rst').read()

import imp
m=imp.load_source('init','src/ladon/__init__.py')
VERSION = '%(major)s.%(minor)s.%(micro)s' % m._version_info

SHORT_DESC = "Serve your python methods to several web service interfaces at once, including JSON-WSP, SOAP and JSON-RPC."
PACKAGES = find_packages('src')

try:
	PACKAGES.remove('chardet_py%d' % other_version)
except:
	pass

install_dir = os.path.dirname(os.path.abspath(__file__))
scripts_dir = os.path.join(install_dir,'scripts')

ladon_command = 'ladon-%d.%d-ctl%s' % (sys.version_info.major,sys.version_info.minor,'.py' if sys.platform.find('win')==0 else '')
scriptfiles = [os.path.join('scripts',ladon_command),os.path.join('scripts','ladon-ctl')]
f_ladon_temp = open(os.path.join(scripts_dir,'ladon-ctl'))
f_ladon_cmd = open(os.path.join(scripts_dir,ladon_command),'w')
f_ladon_cmd.write(f_ladon_temp.read().replace('<INTERPRETER>',sys.executable))
f_ladon_cmd.close()
f_ladon_temp.close()


def walk_dir(dirname):
	files = []
	def detect_svn(fname):
		return fname.find('.svn')==-1
	for f in filter(detect_svn ,map(lambda fname: os.path.join(dirname,fname),os.listdir(dirname))):
		if os.path.isdir(f):
			continue
		files += [f]
	return files

setup(
	name='ladon',
	packages=PACKAGES,
	package_dir={'':'src'},
	version=VERSION,
	description=SHORT_DESC,
	long_description='\n\n'.join([readme, changes]),
	classifiers=[
		'Programming Language :: Python',
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 3',
		'Operating System :: OS Independent',
		'Natural Language :: English',
		'Intended Audience :: Developers',
		'Topic :: Internet :: WWW/HTTP :: WSGI :: Server',
		'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',       
	],
	keywords= ['ladonize', 'soap', 'json', 'shell', 'rpc', 'wsgi'],
	author='Jakob Simon-Gaarde',
	author_email='jakob@simon-gaarde.dk',
	maintainer = 'Jakob Simon-Gaarde',
	maintainer_email = 'jakob@simon-gaarde.dk',
	url='http://ladonize.org',
	install_requires=['jinja2','sphinx','sphinx_bootstrap_theme'],
	requires=['jinja2','sphinx','sphinx_bootstrap_theme'],
	provides=['ladon'],
	license='LGPL3',
	scripts=scriptfiles,
	include_package_data=True,
	zip_safe=False
)



# -*- coding: utf-8 -*-
from wsgiref.util import request_uri
from ladon.types.ladontype import LadonType
import ladon.tools.log as log
import inspect,time
from ladon.tools.threadmanagement import ManagedThread,thread_count,get_thread_info,set_thread_info,thread_running
from ladon.exceptions.service import ServerFault

class TaskInfoResponse(LadonType):
	task_id = {
		'type': int,
		'doc': u'unique integer identifying the task to that has been created'
	}

class TaskProgressResponse(LadonType):
	task_id = {
		'type': int,
		'doc': u'unique integer identifying the task to that has been probed'
	}
	progress = {
		'type': float,
		'doc': u'Floating point value between 0 and 1 denoting the progress in percent'
	}
	duration = {
		'type': float,
		'doc': u'Floating point value that informs how long time in seconds the task as run'
	}
	starttime = {
		'type': float,
		'doc': u'The start time in seconds since the epoch as a floating point number'
	}

taskProgress_args = (int,)
taskProgress_kw = {
	'rtype': TaskProgressResponse
}

taskProgress_kw = {
	'rtype': TaskProgressResponse
}

class TaskRunner(ManagedThread):
	def __init__(self,method,args,kw):
		self.method = method
		self.args = args
		self.kw = kw
		super(TaskRunner,self).__init__()
		
	def run_managed(self):
		argspecs = inspect.getargspec(self.method)
		try:
			if argspecs.keywords!=None:
				res = self.method(*(self.args),**(self.kw))
			else:
				res = self.method(*(self.args))
			set_thread_info(self.thread_id, 'result', res)
		except Exception as e:
			set_thread_info(self.thread_id, 'exception', log.get_traceback())
			log.write("Thread ID: %d (%s) failed with the following exception:\n%s" % (self.thread_id,self.__class__,log.get_traceback()))
			raise e

def taskStarter(self, *args, **kw):
	task_method = getattr(self,"_task_%s" % kw['LADON_METHOD_NAME'])
	tr = TaskRunner(task_method,args,kw)
	kw['update_progress'] = tr.update_progress
	tr.start()
	tir = TaskInfoResponse()
	tir.task_id = tr.thread_id
	return tir

def taskProgress(self, task_id):
	tpr = TaskProgressResponse()
	tpr.task_id = task_id
	tpr.progress = get_thread_info(task_id,'progress')
	if tpr.progress==None:
		raise ServerFault("Task ID: %d does not seem to exist" % task_id)
	tpr.starttime = get_thread_info(task_id,'start')
	stoptime = get_thread_info(task_id,'stop')
	tpr.duration = (stoptime if stoptime else time.time()) - get_thread_info(task_id,'start')
	return tpr

def taskResult(self, task_id):
	exc = get_thread_info(task_id, 'exception')
	if exc:
		raise ServerFault("Error occured while executing task_id: %d" % task_id,exc)
	res = get_thread_info(task_id, 'result')
	if res:
		return res
	else:
		if thread_running(task_id):
			raise ServerFault("Task ID: %d has no result because it is still running" % task_id)
		else:
			raise ServerFault("Task ID: %d has no result and is not running either" % task_id)

